@@ -112,13 +112,22 @@ char *str_dup(const char *s, str_size_t n)
 char *str_concat(char *s, ...)
 {
     va_list ap;
+    char *rv;
+
+    va_start(ap, s);
+    rv = str_concat_va(s, ap);
+    va_end(ap);
+    return rv;
+}
+
+char *str_concat_va(char *s, va_list ap)
+{
     va_list ap_safe;
     int n;
     char *rv;
     char *cp;
     char *ds;
 
-    va_start(ap, s);
     if (s == NULL)
         return NULL;
 
@@ -143,7 +152,6 @@ char *str_concat(char *s, ...)
     *ds = NUL;
 
     /* return target string */
-    va_end(ap);
     return rv;
 }
 
